%Based on paper http://www.cam.wits.ac.za/mfinance/MIF2005/WestGraeme.pdf
%Calibrates alpha,rho,nu parameters for different strikes using input data of market volatilities 

clear all;

global alpha;
global beta;
global rho;
global nu;
global T;
global S;
global r;
global MarketData;


%----------Input parameters ----------------
beta=1; %beta parameter set from backbone or aesthetics
S=22.2; %Spot price
r=0.04; %risk free rate

%market data structure is => Option Maturity in Years/ Strike/ Market Implied Volatility
%Can also be captured by copying data from the "FilteredVols" tab after running 
%the spreadsheet in http://www.quantcode.com/modules/mydownloads/singlefile.php?cid=4&lid=21
MarketData =[ 0.078159208	17.5	0.231583868
0.078159208	20	0.194447616
0.078159208	22.5	0.180948553
0.078159208	25	0.20549825
0.078159208	27.5	0.2286896
0.078159208	30	0.254493656
0.078159208	32.5	0.283817284
0.078159208	35	0.303435636
0.58371  12.00000   0.35215
0.58371  15.00000   0.26899
0.58371  17.00000   0.24382
0.58371  19.50000   0.20943
0.58371  20.00000   0.20439
0.58371  22.00000   0.19634
0.58371  22.50000   0.19318
0.58371  24.50000   0.20162
0.58371  25.00000   0.20537
0.58371  27.00000   0.22337
0.58371  27.50000   0.22825
0.58371  29.50000   0.23799
0.58371  30.00000   0.25288
0.58371  32.00000   0.27194
0.58371  32.50000   0.27655
0.58371  34.50000   0.29436
0.58371  37.00000   0.31516
1.59483  15.00000   0.31353
1.59483  17.50000   0.26695
1.59483  20.00000   0.23042
1.59483  22.50000   0.22082
1.59483  25.00000   0.21535
1.59483  27.50000   0.23587
1.59483  30.00000   0.25090
1.59483  35.00000   0.30947
1.59483  40.00000   0.35199];

alpha=0.2;
rho=0.2;
nu=0.2;

t=MarketData(:,1:2);
data=MarketData(:,3);
wt1 = (1 + 0) ./ sqrt (data); 
F = @leasqrfunc;
dFdp = @dfdp;     % estimated derivative
dp = [0.001; 0.001; 0.001];
pin = [alpha; rho; nu]; 
stol=0.001; 
niter=50;
minstep = [0.01; 0.01; 0.01];
maxstep = [0.8; 0.8; 0.8];
options = [minstep, maxstep];
global Pverbose;
Pverbose(1)=0;
Pverbose(2)=0;

[f1, p1, kvg1, iter1, corp1, covp1, covr1, stdresid1, Z1, r21] = leasqr(t, data, pin, F, stol, niter, wt1, dp, dFdp, options);


alpha=p1(1)
rho=p1(2)
nu=p1(3)

TimesVec=unique(MarketData(:,1)); %list of option expiry times
for i=1:size(TimesVec,1);
T=TimesVec(i); %pick a maturity time
	Option_Maturiy=T
	F=S*exp(r*T); %forward rate for the option maturity
	idx=find(MarketData(:,1)==T);
	X=MarketData(idx,2); %strikes for a given T
	MarketVols=MarketData(idx,3);
	SABRVols=SABRsigma(X,F);
	summary=[X MarketVols SABRVols]
	figure(i);
	plot(X,MarketVols,'+');
    hold all;
    plot(X,SABRVols)
    hold off;
	title(['Maturity=',num2str(T),'years']);
	xlabel('Strike');
	ylabel('Volatility');
end

TimesVec=unique(MarketVols(:,1));
T=TimesVec(1);
F=S*exp(r*T);
t=MarketData(:,1:2);
idx=find(MarketVols(:,1)==T);
X=MarketData(idx,2);
market_impliedvols=MarketData(idx,3)
SABR_vols=SABRsigma(X,F)
x=market_impliedvols
y=SABR_vols
%return;

X=S-10:S+10;
X=X';
Tvec=0.1:1.5;
Tvec=Tvec';
outmat=zeros(size(X),size(Tvec));
for i=1:size(Tvec),
	T=Tvec(i);
	F=exp(r*T)*S;
	outmat(:,i)=SABRsigma(X,F);
end
figure(4)
surf(outmat);
% return;
% 	
% alpha = 0.18346
% rho = 0.067177
% nu = 1.029
% T=TimesVec(3);
% F=S*exp(r*T);
% t=MarketData(:,1:2);
% idx=find(MarketVols(:,1)==T);
% X=MarketData(idx,2);
% market_impliedvols=MarketData(idx,3)
% SABR_vols=SABRsigma(X,F)
% return;
